#include "bed_file.hpp"

#define LINE 65536

bool Bed::operator<(const Bed &one) const
{
  if (this->chromosome < one.chromosome)
  {
    return true;
  }
  if (this->chromosome > one.chromosome)
  {
    return false;
  }
  if (this->start < one.start)
  {
    return true;
  }
  if (this->start > one.start)
  {
    return false;
  }
  if (this->end < one.end)
  {
    return true;
  }
  if (this->end > one.end)
  {
    return false;
  }
  return false;
}

template<class T>
bool Bed::operator<(const T &one) const
{
  if (this->chromosome < one.chromosome)
  {
    return true;
  }
  if (this->chromosome > one.chromosome)
  {
    return false;
  }
  if (this->end <= one.position)
  {
    return true;
  }
  return false;
}

template<class T>
bool Bed::operator==(const T &one) const
{
  if (this->chromosome == one.chromosome && this->start<=one.position && this->end>one.position)
  {
    return true;
  }
  else
  {
    return false;
  }
}

template<class T>
bool Bed::operator>(const T &one) const
{
  if (this->chromosome < one.chromosome)
  {
    return false;
  }
  else if (this->chromosome>one.chromosome)
  {
    return true;
  }
  else if (this->start > one.position)
  {
    return true;
  }
  else
  {
    return false;
  }
}

bool Point::operator<(const Point &one) const
{
  if (this->chromosome<one.chromosome)
  {
    return true;
  }
  else if (this->chromosome>one.chromosome)
  {
    return false;
  }
  else if (this->position < one.position)
  {
    return true;
  }
  else
  {
    return false;
  }
}

bool Point::operator<(const Bed &one) const
{
  return one>*this;
}

bool Point::operator==(const Bed &one) const
{
  return one==*this;
}

bool Point::operator>(const Bed &one) const
{
  return one<*this;
}

bool read_cgi(string cgi_file,
              vector<Bed> &cgi)
{
  fstream ifs;
  stringstream sstrm;
  char buff[LINE];
  Bed cgi_bed;
  cgi.clear();
  ifs.open(cgi_file.c_str(), ios_base::in);
  if (!ifs.is_open())
  {
    cout << "Error: cannot open file " << cgi_file <<endl;
    return false;
  }
  while(true)
  {
    ifs.getline(buff, LINE-1);
    if (!ifs)
    {
      break;
    }
    if (buff[0]=='#')
      continue;
    sstrm.str("");
    sstrm.clear();
    sstrm<<buff;
    sstrm>>cgi_bed.chromosome;
    sstrm>>cgi_bed.start;
    sstrm>>cgi_bed.end;
    sstrm>>cgi_bed.name;
    if (!sstrm)
    {
      cout<<"Warning: wrong format encountered when reading bed file"<<endl<<"wrong sentence is: "<<buff<<endl;
      continue;
    }
    cgi.push_back(cgi_bed);
  }
  sort(cgi.begin(), cgi.end());
  ifs.close();
  return true;
}

bool read_probe_ann(string &ann_file, vector<Point> &ann, map<string, size_t> &map_probe)
{
  string probe_id;
  string chromosome;
  size_t pos;
  string anno;
  char buff[LINE];
  stringstream sstrm;
  Point tmp_pnt;
  ann.clear();
  fstream ifs;
  ifs.open(ann_file.c_str(), ios_base::in);
  if (!ifs.is_open())
  {
    printf("Error: cannot open file %s\n", ann_file.c_str());
    return false;
  }
  while(true)
  {
    ifs.getline(buff, LINE-1);
    if (!ifs)
    {
      break;
    }
    if (buff[0]=='#')
      continue;
    sstrm.clear();
    sstrm.str("");
    sstrm<<buff;
    sstrm>>probe_id>>chromosome>>pos>>anno;
    if (!sstrm || pos==0)
    {
      cout<<"Warnning: wrong format encountered when reading probe annotation"<<endl<<"Wrong sentence is: "<<buff<<endl;
      continue;
    }
    tmp_pnt.chromosome=chromosome;
    tmp_pnt.position=pos;
    tmp_pnt.name=probe_id;
    tmp_pnt.annotation=anno;
    ann.push_back(tmp_pnt);
  }
  sort(ann.begin(), ann.end());
  for(pos=0; pos<ann.size(); pos++)
  {
    probe_id=ann[pos].name;
    map_probe.insert(map<string, size_t>::value_type(make_pair(probe_id, pos)));
  }
  return true;
}

void cgi_probe(vector<Bed> &cgi,
               vector<Point> &ann_w,
               Data_description &data_des,
               map<string, size_t> &map_probe,
               map<size_t, vector<string> > &cgi_probe)
{
  size_t i; 
  size_t j;
  vector<string> emp_vec;
  Point tmp_pb;
  string probe_name;
  vector<Point> ann;
  for (i=0; i<data_des.probe_index.size(); i++)
  {
    probe_name=data_des.probe_index.get(i);
    j=map_probe[probe_name];
    tmp_pb.chromosome=ann_w[j].chromosome;
    tmp_pb.position=ann_w[j].position;
    tmp_pb.name=ann_w[j].name;
    tmp_pb.annotation=ann_w[j].annotation;
    ann.push_back(tmp_pb);
  }
  sort(ann.begin(), ann.end());
  j=0;
  for (i=0; i<cgi.size(); i++)
  {
    if (j==ann.size())
    {
      break;
    }
    if (cgi[i]==ann[j])
    {
      if (cgi_probe.find(i)==cgi_probe.end())
      {
        cgi_probe.insert(map<size_t, vector<string> >::value_type(make_pair(i,emp_vec)));
      }
      cgi_probe[i].push_back(ann[j].name);
      while(cgi[i]==ann[j])
      {
        j++;
        if (j==ann.size())
        {
          break;
        }
        if (ann[j]>cgi[i])
        {
          break;
        }
        cgi_probe[i].push_back(ann[j].name);
      }
    }
    else if (cgi[i]>ann[j])
    {
      while(true)
      {
        j++;
        if (j==ann.size())
        {
          break;
        }
        if (ann[j]>cgi[i])
        {
          break;
        }
        if (ann[j]==cgi[i])
        {
          if (cgi_probe.find(i)==cgi_probe.end())
          {
            cgi_probe.insert(map<size_t, vector<string> >::value_type(make_pair(i,emp_vec)));
          }
          cgi_probe[i].push_back(ann[j].name);
        }
      }
    }
  }
}
