#include "cgi_analysis.hpp"

void cgi_analysis_thread(Data_matrix *data,
                         size_t remainder,
                         size_t nproc,
                         Data_description *data_des,
                         CGI_description *cgi_des,
                         CGI_result *cgi_res)
{
  gsl_vector *obs;
  gsl_matrix *par_h0, *par_h1;
  vector<double> obs_data;
  vector<size_t> obs_sample;
  vector<size_t> obs_position;
  vector<string> obs_probe;
  size_t covariable_number;
  if (data->covariable==0)
  {
    covariable_number=0;
  }
  else
  {
    covariable_number=data->covariable->size2;
  }
  size_t i;
  size_t n;
  size_t j;
  string probe_id;
  size_t probe_index;
  size_t k;
  size_t sample_number;
  size_t sample_index;
  string sample_id;
  string group_id;
  size_t group_index;
  sample_number=data->data->size2;
  size_t group_number;
  group_number=data_des->group_index.size();
  for (i=remainder; i<cgi_des->cgi.size(); i+=nproc)
  {
    obs_data.clear();
    obs_sample.clear();
    obs_position.clear();
    obs_probe.clear();
    n=cgi_des->cgi_probe[i].size();
    if (n<2)
    {
      continue;
    }
    for (j=0; j<n; j++)
    {
      probe_id=cgi_des->cgi_probe[i][j];
      probe_index=data_des->probe_index.get(probe_id);
      for(k=0; k<sample_number; k++)
      {
        if (gsl_matrix_get(data->valid, probe_index, k)<0.0)
        {
          continue;
        }
        obs_data.push_back(gsl_matrix_get(data->data, probe_index, k)); 
        obs_sample.push_back(k);
        obs_position.push_back(j);
        obs_probe.push_back(probe_id);
      }
    }
    obs=gsl_vector_alloc(obs_data.size());
    if (data->continuous==0)
    {
      par_h0=gsl_matrix_alloc(obs_data.size(), n*(covariable_number+1));
      par_h1=gsl_matrix_alloc(obs_data.size(), n*(covariable_number+group_number));
    }
    else
    {
      par_h0=gsl_matrix_alloc(obs_data.size(), n*(covariable_number+1));
      par_h1=gsl_matrix_alloc(obs_data.size(), n*(covariable_number+2));
    }
    gsl_matrix_set_all(par_h0, 0.0);
    gsl_matrix_set_all(par_h1, 0.0);
    size_t pos;
    for (j=0; j<obs_data.size(); j++)
    {
      gsl_vector_set(obs, j, obs_data[j]);
      probe_id=obs_probe[j];
      probe_index=data_des->probe_index.get(probe_id);
      pos=obs_position[j];
      sample_index=obs_sample[j];
      sample_id=data_des->sample_index.get(sample_index);
      group_id=data_des->sample_group.get_group(sample_id);
      group_index=data_des->group_index.get(group_id);
      for (k=0; k<covariable_number; k++)
      {
        gsl_matrix_set(par_h0, j, pos*(covariable_number+1)+k, gsl_matrix_get(data->covariable, sample_index, k));
        if (data->continuous==0)
        {
          gsl_matrix_set(par_h1, j, pos*(covariable_number+group_number)+k, gsl_matrix_get(data->covariable, sample_index, k));
        }
        else
        {
          gsl_matrix_set(par_h1, j, pos*(covariable_number+2)+k, gsl_matrix_get(data->covariable, sample_index, k));
        }
      }
      gsl_matrix_set(par_h0, j, pos*(covariable_number+1)+covariable_number, 1.0);
      if (data->continuous==0)
      {
        gsl_matrix_set(par_h1, j, pos*(covariable_number+group_number)+covariable_number+group_index, 1.0);
      }
      else
      {
        gsl_matrix_set(par_h1, j, pos*(covariable_number+2)+covariable_number, 1.0);
        gsl_matrix_set(par_h1, j, pos*(covariable_number+2)+covariable_number+1, gsl_matrix_get(data->continuous, sample_index, 0));
      }
    }
    double pval;
    pval=ancova(obs, par_h0, par_h1);
    gsl_vector_set(cgi_res->pvalue, i, pval);
    gsl_matrix_free(par_h0);
    gsl_matrix_free(par_h1);
    gsl_vector_free(obs);
  }
}

void fdr_range_cgi(CGI_result &cgi_res,
                   CGI_description &cgi_des,
                   Data_description &data_des)
{
  size_t i;
  size_t j;
  size_t k;
  string cgi_name;
  string probe_name;
  size_t pi;
  size_t num;
  double range;
  double tmp_dbl;
  double former_fdr;
  Order tmp_odr;
  vector<Order> pvals;
  stringstream sstrm;
  num=0;
  for (i=0; i<cgi_res.pvalue->size; i++)
  {
    tmp_dbl=gsl_vector_get(cgi_res.pvalue, i);
    if (tmp_dbl<1+1e-6)
    {
      num++;
      sstrm.str("");
      sstrm.clear();
      sstrm<<i;
      sstrm>>cgi_name;
      tmp_odr.str=cgi_name;
      tmp_odr.dbl=tmp_dbl;
      pvals.push_back(tmp_odr);
    }
  }
  sort(pvals.begin(), pvals.end());
  former_fdr=0.0;
  for (i=0; i<pvals.size(); i++)
  {
    sstrm.str("");
    sstrm.clear();
    sstrm<<pvals[i].str;
    sstrm>>j;
    tmp_dbl=pvals[i].dbl;
    tmp_dbl=tmp_dbl*num/(i+1);
    if (former_fdr>tmp_dbl)
    {
      tmp_dbl=former_fdr;
    }
    else
    {
      former_fdr=tmp_dbl;
    }
    if (tmp_dbl>1.0)
    {
      tmp_dbl=1.0;
    }
    gsl_vector_set(cgi_res.fdr, j, tmp_dbl);
  }
}

void cgi_analysis(Data_matrix *data,
                  size_t nproc,
                  string &cgi_filename,
                  Data_description &data_des,
                  CGI_result &cgi_res,
                  CGI_description &cgi_des)
{
  time_out();
  printf("     Analyzing differentially methylated CGI\n");
  time_out();
  printf("     Calculating pvalues for CGI\n");
  cgi_res.pvalue=gsl_vector_alloc(cgi_des.cgi.size());
  cgi_res.fdr=gsl_vector_alloc(cgi_des.cgi.size());
  gsl_vector_set_all(cgi_res.pvalue, 2.0);
  gsl_vector_set_all(cgi_res.fdr, 2.0);
  boost::thread_group *thrdg;
  boost::thread *thrd;
  size_t remainder;
  thrdg=new boost::thread_group;
  for (remainder=0; remainder<nproc; remainder++)
  {
    thrd=new boost::thread(cgi_analysis_thread, data, remainder, nproc, &data_des, &cgi_des, &cgi_res);
    thrdg->add_thread(thrd);
  }
  thrdg->join_all();
  time_out();
  printf("       Finding fdr in CGI analysis\n");
  fdr_range_cgi(cgi_res, cgi_des, data_des);
}
