#include "dmr_analysis.hpp"

#define LENGTH 200
#define THRESHOLD 0.05

inline size_t probe_distance(vector<Point> &ann, size_t i, size_t j)
{
  if (ann[i].chromosome!=ann[j].chromosome)
  {
    return LENGTH+1;
  }
  else
  {
    return ann[j].position-ann[i].position;
  }
}

void find_region(vector<Point> &ann, map<string, size_t> &map_probe, CGI_description &cgi_des)
{
  size_t i;
  size_t j;
  Bed tmp_bed;
  vector<string> tmp_vec;
  for(i=0; i<ann.size()-1; i++)
  {
    tmp_vec.clear();
    tmp_vec.push_back(ann[i].name);
    j=i+1;
    while(j<ann.size() && probe_distance(ann, i, j)<=LENGTH)
    {
      tmp_vec.push_back(ann[j].name);
      j++;
    }
    if (tmp_vec.size()>1)
    {
      tmp_bed.chromosome=ann[i].chromosome;
      tmp_bed.start=ann[i].position;
      tmp_bed.end=ann[j-1].position+1;
      cgi_des.cgi.push_back(tmp_bed);
      cgi_des.cgi_probe.insert(map<size_t, vector<string> >::value_type(make_pair(cgi_des.cgi.size()-1, tmp_vec)));
    }
  }
}

void dmr_analysis(Data_description &data_des, Probe_result &probe_res, vector<Point> &ann, map<string, size_t> &map_probe, DMR_result &dmr_res)
{
  size_t i;
  size_t j;
  double sum;
  size_t s;
  size_t e;
  Bed tmp_bed;
  string tmp_str;
  size_t tmp_index;
  vector<string> tmp_vec;
  vector<size_t> dmr_vec;
  set<string> tmp_set;
  set<string>::iterator its;
  CGI_description cgi_des;
  time_out();
  printf("     Analyzing differentially methylated regions\n");
  find_region(ann, map_probe, cgi_des);
  for(i=0; i<cgi_des.cgi.size(); i++)
  {
    sum=0.0;
    for(j=0; j<cgi_des.cgi_probe[i].size(); j++)
    {
      tmp_str=cgi_des.cgi_probe[i][j];
      tmp_index=data_des.probe_index.get(tmp_str);
      sum+=log(1e-300+gsl_vector_get(probe_res.fdr, tmp_index));
    }
    if (sum/cgi_des.cgi_probe[i].size()<log(THRESHOLD))
    {
      dmr_vec.push_back(i);
    }
  }
  for(i=0; i<dmr_vec.size(); i++)
  {
    if (tmp_bed.chromosome != cgi_des.cgi[dmr_vec[i]].chromosome || tmp_bed.end<cgi_des.cgi[dmr_vec[i]].start)
    {
      if (!tmp_set.empty())
      {
        dmr_res.dmr.push_back(tmp_bed);
        tmp_vec.clear();
        for(its=tmp_set.begin(); its!=tmp_set.end(); its++)
        {
          tmp_str=*its;
          tmp_vec.push_back(tmp_str);
        }
        dmr_res.dmr_probe.insert(map<size_t, vector<string> >::value_type(make_pair(dmr_res.dmr.size()-1, tmp_vec)));
      }
      tmp_set.clear();
      tmp_bed.chromosome=cgi_des.cgi[dmr_vec[i]].chromosome;
      tmp_bed.start=cgi_des.cgi[dmr_vec[i]].start;
      tmp_bed.end=cgi_des.cgi[dmr_vec[i]].end;
      for(j=0; j<cgi_des.cgi_probe[dmr_vec[i]].size(); j++)
      {
        tmp_set.insert(cgi_des.cgi_probe[dmr_vec[i]][j]);
      }
    }
    else
    {
      tmp_bed.end=cgi_des.cgi[dmr_vec[i]].end;
      for  (j=0; j<cgi_des.cgi_probe[dmr_vec[i]].size(); j++)
      {
        tmp_set.insert(cgi_des.cgi_probe[dmr_vec[i]][j]);
      }
    }
  }
  if (!tmp_set.empty())
  {
    dmr_res.dmr.push_back(tmp_bed);
    tmp_vec.clear();
    for(its=tmp_set.begin(); its!=tmp_set.end(); its++)
    {
      tmp_str=*its;
      tmp_vec.push_back(tmp_str);
    }
    dmr_res.dmr_probe.insert(map<size_t, vector<string> >::value_type(make_pair(dmr_res.dmr.size()-1, tmp_vec)));
  }
  time_out();
  printf("     Writing DMR analysis result\n");
}

void dmr_analysis_2(Data_description &data_des, Data_matrix *data, Parameter &param, vector<Point> &ann, map<string, size_t> &map_probe, DMR_result &dmr_res)
{
  CGI_description cgi_des;
  Bed tmp_bed;
  vector<string> tmp_vec;
  set<string> tmp_set;
  set<string>::iterator its;
  string tmp_str;
  time_out();
  printf("     Analyzing differentially methylated regions\n");
  find_region(ann, map_probe, cgi_des);
  CGI_result cgi_res;
  cgi_res.pvalue=gsl_vector_alloc(cgi_des.cgi.size());
  cgi_res.fdr=gsl_vector_alloc(cgi_des.cgi.size());
  size_t remainder;
  size_t nproc=param.nproc;
  boost::thread_group *thrdg;
  boost::thread *thrd;
  thrdg=new boost::thread_group;
  for(remainder=0; remainder<nproc; remainder++)
  {
    thrd=new boost::thread(cgi_analysis_thread, data, remainder, nproc, &data_des, &cgi_des, &cgi_res);
    thrdg->add_thread(thrd);
  }
  thrdg->join_all();
  fdr_range_cgi(cgi_res, cgi_des, data_des);
  vector<size_t> dmr_vec;
  size_t i;
  size_t j;
  double sum_p, sum_f;
  double tmp_dbl;
  for (i=0; i<cgi_des.cgi.size(); i++)
  {
    sum_p=0.0;
    sum_f=0.0;
    tmp_dbl=gsl_vector_get(cgi_res.pvalue, i);
    if (tmp_dbl<0 || tmp_dbl>1)
      tmp_dbl=1.0;
    else
      tmp_dbl+=1e-300;
    sum_p+=log(tmp_dbl);
    tmp_dbl=gsl_vector_get(cgi_res.fdr, i);
    if (tmp_dbl<0 || tmp_dbl>1)
      tmp_dbl=1.0;
    else
      tmp_dbl+=1e-300;
    sum_f+=log(tmp_dbl);
    if (sum_p/cgi_des.cgi_probe[i].size() < log(param.pvalue_threshold) && sum_f/cgi_des.cgi_probe[i].size() < log(param.fdr_threshold))
    {
      dmr_vec.push_back(i);
    }
  }
  for(i=0; i<dmr_vec.size(); i++)
  {
    if (tmp_bed.chromosome != cgi_des.cgi[dmr_vec[i]].chromosome || tmp_bed.end<cgi_des.cgi[dmr_vec[i]].start)
    {
      if (!tmp_set.empty())
      {
        dmr_res.dmr.push_back(tmp_bed);
        tmp_vec.clear();
        for(its=tmp_set.begin(); its!=tmp_set.end(); its++)
        {
          tmp_str=*its;
          tmp_vec.push_back(tmp_str);
        }
        dmr_res.dmr_probe.insert(map<size_t, vector<string> >::value_type(make_pair(dmr_res.dmr.size()-1, tmp_vec)));
      }
      tmp_set.clear();
      tmp_bed.chromosome=cgi_des.cgi[dmr_vec[i]].chromosome;
      tmp_bed.start=cgi_des.cgi[dmr_vec[i]].start;
      tmp_bed.end=cgi_des.cgi[dmr_vec[i]].end;
      for(j=0; j<cgi_des.cgi_probe[dmr_vec[i]].size(); j++)
      {
        tmp_set.insert(cgi_des.cgi_probe[dmr_vec[i]][j]);
      }
    }
    else
    {
      tmp_bed.end=cgi_des.cgi[dmr_vec[i]].end;
      for  (j=0; j<cgi_des.cgi_probe[dmr_vec[i]].size(); j++)
      {
        tmp_set.insert(cgi_des.cgi_probe[dmr_vec[i]][j]);
      }
    }
  }
  if (!tmp_set.empty())
  {
    dmr_res.dmr.push_back(tmp_bed);
    tmp_vec.clear();
    for(its=tmp_set.begin(); its!=tmp_set.end(); its++)
    {
      tmp_str=*its;
      tmp_vec.push_back(tmp_str);
    }
    dmr_res.dmr_probe.insert(map<size_t, vector<string> >::value_type(make_pair(dmr_res.dmr.size()-1, tmp_vec)));
  }
  time_out();
  printf("     Writing DMR result\n");
}
