#include "normalize.hpp"

bool normalize_function(Data_matrix *data,
                        size_t col,
                        gsl_vector *ref_quantile,
                        gsl_vector *ref_value,
                        gsl_vector *value)
{
  if (ref_quantile->size!=ref_value->size || ref_quantile->size!=value->size)
  {
    printf("Error: quantile and value does not match\n");
    return false;
  }
  size_t i;
  double tmp_double;
  double tmp_valid;
  size_t j;
  for (i=0; i<data->data->size1; i++)
  {
    tmp_double=gsl_matrix_get(data->data,i,col);
    tmp_valid=gsl_matrix_get(data->valid, i, col);
    if (tmp_valid<0.0)
    {
      continue;
    }
    for(j=0; j<ref_value->size-1; j++)
    {
      if (tmp_double>gsl_vector_get(value,j)-1e-10 && tmp_double<gsl_vector_get(value,j+1)+1e-10)
      {
        tmp_double=gsl_vector_get(ref_value,j)
                   +(gsl_vector_get(ref_value,j+1)-gsl_vector_get(ref_value,j))
                   *(tmp_double-gsl_vector_get(value,j))
                   /(gsl_vector_get(value,j+1)-gsl_vector_get(value,j));
        gsl_matrix_set(data->data,i,col,tmp_double);
        break;
      }
    }
  }
  return true;
}

bool normalize_function_thread(Data_matrix *data,
                               size_t remainder,
                               size_t nproc,
                               gsl_vector *ref_quantile,
                               gsl_vector *ref_value,
                               gsl_matrix *value)
{
  if (ref_quantile->size != ref_value->size || ref_quantile->size != value->size1)
  {
    printf("Error: quantile and value are not match\n");
    return false;
  }
  if (value->size2 != data->data->size2)
  {
    printf("Error: sample number does not match\n");
    return false;
  }
  gsl_vector *value_v;
  value_v=gsl_vector_alloc(ref_quantile->size);
  size_t i;
  for (i=remainder; i<data->data->size2; i+=nproc)
  {
    gsl_matrix_get_col(value_v, value, i);
    normalize_function(data,
                       i,
                       ref_quantile,
                       ref_value,
                       value_v);
  }
  gsl_vector_free(value_v);
  return true;
}

bool find_quantile_thread(Data_matrix *data,
                          size_t remainder,
                          size_t nproc,
                          gsl_vector *ref_quantile,
                          gsl_matrix *value)
{
  if (data->data->size2!=value->size2)
  {
    printf("Error: sample number does not match\n");
    return false;
  }
  if (value->size1 != ref_quantile->size)
  {
    printf("Error: quantile number does not match\n"); 
    return false;
  }
  size_t valid;
  gsl_vector *betas;
  size_t i;
  size_t j;
  double b;
  for (i=remainder; i<data->data->size2; i+=nproc)
  {
    valid=0;
    for (j=0; j<data->data->size1; j++)
    {
      b=gsl_matrix_get(data->valid, j, i);
      if (b>0)
      {
        valid++;
      }
    }
    betas=gsl_vector_alloc(valid);
    valid=0;
    for (j=0; j<data->data->size1; j++)
    {
      b=gsl_matrix_get(data->valid, j, i);
      if(b>0)
      {
        gsl_vector_set(betas, valid , gsl_matrix_get(data->data, j, i));
        valid++;
      }
    }
    gsl_sort_vector(betas);
    for (j=0; j<ref_quantile->size; j++)
    {
      b=gsl_stats_quantile_from_sorted_data(betas->data, betas->stride, betas->size, gsl_vector_get(ref_quantile, j));
      gsl_matrix_set(value, j, i, b);
    }
    gsl_vector_free(betas);
  }
  return true;
}

bool average_quantile(gsl_matrix *value,
                      gsl_vector *ref_value)
{
  if (ref_value->size != value->size1)
  {
    printf("Error: quantile number does not match\n");
    return false;
  }
  size_t i;
  double mean;
  gsl_vector *row;
  row=gsl_vector_alloc(value->size2);
  for (i=0; i<value->size1; i++)
  {
    gsl_matrix_get_row(row, value, i);
    mean=gsl_stats_mean(row->data, row->stride, row->size);
    gsl_vector_set(ref_value, i, mean);
  }
  gsl_vector_free(row);
  return true;
}

bool normalize(Data_matrix *data,
               size_t nproc)
{
  time_out();printf("     ");
  printf("Normalizing data\n");
  size_t remainder;
  gsl_vector *n;
  boost::thread_group *thrdg;
  boost::thread *thrd;
  size_t i;
  double quantile;
  gsl_vector *ref_quantile;
  ref_quantile=gsl_vector_alloc(15);
  for (i=0; i<15; i++)
  {
    quantile=0.07*i+0.01;
    gsl_vector_set(ref_quantile, i, quantile);
  }
  gsl_matrix *value;
  value=gsl_matrix_alloc(15, data->data->size2);
  time_out();
  printf("       Finding quantile\n");
  thrdg=new boost::thread_group;
  for (i=0; i<nproc; i++)
  {
    thrd=new boost::thread(find_quantile_thread, data, i, nproc, ref_quantile, value);
    thrdg->add_thread(thrd);
  }
  thrdg->join_all();
  delete thrdg;
  gsl_vector *ref_value;
  ref_value=gsl_vector_alloc(ref_quantile->size);
  average_quantile(value, ref_value);
  thrdg=new boost::thread_group;
  time_out();
  printf("       Correcting quantile\n");
  for (i=0; i<nproc; i++)
  {
    thrd=new boost::thread(normalize_function_thread,
                           data,
                           i,
                           nproc,
                           ref_quantile,
                           ref_value,
                           value);
    thrdg->add_thread(thrd);
  }
  thrdg->join_all();
  delete thrdg;
  gsl_vector_free(ref_value);
  gsl_matrix_free(value);
  gsl_vector_free(ref_quantile);
  return true;
}
