#include "probe_analysis.hpp"

void probe_analysis_thread(Data_matrix *data,
                           size_t remainder,
                           size_t nproc,
                           Data_description *data_description,
                           Probe_result *probe_result)
{
  size_t i;
  size_t j;
  size_t k;
  size_t l, m, group_num;
  size_t covariable_number;
  size_t valid;
  vector<pair<double, double> > ave;
  for (i=0; i<data_description->group_index.size(); i++)
  {
    ave.push_back(make_pair(1.0,1.0));
  }
  double tmp_dbl;
  double tmp_min, tmp_max, range;
  string tmp_str;
  if (data->covariable==0)
  {
    covariable_number=0;
  }
  else
  {
    covariable_number=data->covariable->size2;
  }
  gsl_vector *obs;
  gsl_matrix *par_h0;
  gsl_matrix *par_h1;
  for(i=remainder; i<data->data->size1; i+=nproc)
  {
    k=0;
    for (j=0; j<data->data->size2; j++)
    {
      tmp_dbl=gsl_matrix_get(data->valid, i, j);
      if (tmp_dbl>0.0)
      {
        k++;
      }
    }
    obs=gsl_vector_alloc(k);
    if (data->continuous==0)
    {
      par_h0=gsl_matrix_alloc(k, covariable_number+1);
      par_h1=gsl_matrix_alloc(k, covariable_number+data_description->group_index.size());
      gsl_matrix_set_all(par_h1, 0.0);
    }
    else
    {
      par_h0=gsl_matrix_alloc(k, covariable_number+1);
      par_h1=gsl_matrix_alloc(k, covariable_number+2);
      gsl_matrix_set_all(par_h1, 0.0);
    }
    l=0;
    for (j=0; j<data->data->size2; j++)
    {
      if (gsl_matrix_get(data->valid, i, j)>0.0)
      {
        gsl_vector_set(obs, l, gsl_matrix_get(data->data, i, j));
        for (m=0; m<covariable_number; m++)
        {
          gsl_matrix_set(par_h0, l, m, gsl_matrix_get(data->covariable, j, m));
          gsl_matrix_set(par_h1, l, m, gsl_matrix_get(data->covariable, j, m));
        }
        gsl_matrix_set(par_h0, l, covariable_number, 1.0);
        tmp_str=data_description->sample_index.get(j);
        if (data->continuous==0)
        {
          tmp_str=data_description->sample_group.get_group(tmp_str);
          group_num=data_description->group_index.get(tmp_str);
          gsl_matrix_set(par_h1, l, covariable_number+group_num, 1.0);
        }
        else
        {
          gsl_matrix_set(par_h1, l, covariable_number, 1.0);
          gsl_matrix_set(par_h1, l, covariable_number+1, gsl_matrix_get(data->continuous, j, 0));
        }
        l++;
      }
    }
    gsl_vector_set(probe_result->pvalue, i, ancova(obs, par_h0, par_h1));
    if (gsl_vector_get(probe_result->pvalue, i)> 1.0)
    {
      cout << "Warning: too few observations for probe" << data_description->probe_index.get(i)<<endl;
    }
    for (j=0; j<data_description->group_index.size(); j++)
    {
      ave[j].first=0.0;
      ave[j].second=0.0;
    }
    for (j=0; j<data_description->sample_index.size(); j++)
    {
      if (gsl_matrix_get(data->valid, i, j)<0.0)
      {
        continue;
      }
      tmp_str=data_description->sample_index.get(j);
      tmp_str=data_description->sample_group.get_group(tmp_str);
      valid=data_description->group_index.get(tmp_str);
      ave[valid].first+=gsl_matrix_get(data->data, i, j);
      ave[valid].second+=1.0;
    }
    for (j=0; j<ave.size(); j++)
    {
      if (ave[j].second<1.0)
      {
        ave[j].first=0.0;
        cout<<"Warning: no valid beta value of probe "<<data_description->probe_index.get(i)<<" in group "<<data_description->group_index.get(j)<<endl;
      }
      else
      {
        ave[j].first/=ave[j].second;
      }
      gsl_matrix_set(probe_result->average, i, j, exp(ave[j].first)/(1+exp(ave[j].first)));
    }
    tmp_max=0.0;
    tmp_min=1.0;
    for (j=0; j<probe_result->average->size2; j++)
    {
      tmp_dbl=gsl_matrix_get(probe_result->average, i, j);
      if (tmp_dbl<tmp_min)
        tmp_min=tmp_dbl;
      if (tmp_dbl>tmp_max)
        tmp_max=tmp_dbl;
    }
    gsl_vector_set(probe_result->range, i, tmp_max-tmp_min);
    gsl_vector_free(obs);
    gsl_matrix_free(par_h0);
    gsl_matrix_free(par_h1);
  }
}

void fdr_range(Data_matrix *data, Probe_result &probe_result, Data_description &data_description)
{
  double fdr;
  size_t group_num=data_description.group_index.size();
  size_t valid;
  Order tmp_odr;
  double tmp_dbl;
  vector<Order> probe_pvalue;
  string tmp_str;
  size_t i;
  size_t j;
  gsl_vector_set_all(probe_result.fdr, 1.0);
  for (i=0; i<probe_result.pvalue->size; i++)
  {
    tmp_dbl=gsl_vector_get(probe_result.pvalue, i);
    if (tmp_dbl<1.0+1e-6)
    {
      tmp_odr.dbl=tmp_dbl;
      tmp_odr.str=data_description.probe_index.get(i);
      probe_pvalue.push_back(tmp_odr);
    }
  }
  valid=probe_pvalue.size();
  sort(probe_pvalue.begin(), probe_pvalue.end());
  tmp_dbl=0.0;
  for (i=0; i<probe_pvalue.size(); i++)
  {
    if (tmp_dbl>probe_pvalue[i].dbl*valid/(i+1))
    {
      probe_pvalue[i].dbl=tmp_dbl;
    }
    else
    {
      probe_pvalue[i].dbl=probe_pvalue[i].dbl*valid/(i+1);
    }
    if (probe_pvalue[i].dbl>1.0)
    {
      probe_pvalue[i].dbl=1.0;
    }
    tmp_dbl=probe_pvalue[i].dbl;
  }
  for (i=0; i<probe_pvalue.size(); i++)
  {
    j=data_description.probe_index.get(probe_pvalue[i].str);
    gsl_vector_set(probe_result.fdr, j, probe_pvalue[i].dbl);
  }
}

bool probe_analysis(Data_matrix *data,
                   size_t nproc,
                   Data_description &data_description,
                   Probe_result &probe_result)
{
  size_t remainder;
  time_out();
  printf("     Probe analysis\n");
  size_t nrow;
  size_t ncol;
  nrow=data->data->size1;
  ncol=data->data->size2;
  probe_result.average=gsl_matrix_alloc(nrow, data_description.group_index.size());
  probe_result.pvalue=gsl_vector_alloc(nrow);
  probe_result.range=gsl_vector_alloc(nrow);
  probe_result.fdr=gsl_vector_alloc(nrow);
  gsl_matrix_set_all(probe_result.average, 2.0);
  gsl_vector_set_all(probe_result.pvalue, 2.0);
  gsl_vector_set_all(probe_result.fdr, 1.0);
  gsl_vector_set_all(probe_result.range, -1.0);
  time_out();
  printf("       Calculate pvalue\n");
  boost::thread_group *thrdg;
  boost::thread *thrd;
  thrdg=new boost::thread_group;
  for (remainder=0; remainder<nproc; remainder++)
  {
    thrd=new boost::thread(probe_analysis_thread, data, remainder, nproc, &data_description, &probe_result);
    thrdg->add_thread(thrd);
  }
  thrdg->join_all();
  delete thrdg;
  time_out();
  printf("       Calculate fdr and range\n");
  fdr_range(data, probe_result, data_description);
  return true;
}
