#include "write_file.hpp"

bool write_probe_result(Data_description &data_des,
                        Probe_result &probe_result,
                        vector<Point> &ann,
                        map<string, size_t> &map_probe,
                        Parameter &parameter)
{
  time_out();
  printf("     Write probe analysis result\n");
  size_t i;
  size_t j;
  fstream ofs;
  double pvalue;
  double range;
  double fdr;
  ofs.open(parameter.probe_test_file.c_str(), ios_base::out);
  if (!ofs.is_open())
  {
    printf("Error: cannot open file %s\n", parameter.probe_test_file.c_str());
    return false;
  }
  ofs<<"ID\tPvalue\tFDR\tRange\t";
  for (i=0; i<data_des.group_index.size(); i++)
  {
    ofs<<data_des.group_index.get(i)<<"\t";
  }
  ofs<<"CHR\tLOC\tANN\n";
  string probe_name;
  FILE *of_tmp;
  vector<FILE*> of_bgs;
  size_t group_num=data_des.group_index.size();
  string tmp_str;
  string fname;
  for(i=0; i<group_num; i++)
  {
    tmp_str=data_des.group_index.get(i);
    fname=parameter.probe_prefix+"_"+tmp_str+".BedGraph";
    if(!(of_tmp=fopen(fname.c_str(), "w")))
    {
      cout<<"Error: can not open file "<<fname<<endl;
    }
    of_bgs.push_back(of_tmp);
  }
  for (i=0; i<probe_result.pvalue->size; i++)
  {
    probe_name=data_des.probe_index.get(i);
    if ((probe_name[0]=='r'||ann[map_probe[probe_name]].annotation[0]=='*') && parameter.snp)
      continue;
    pvalue=gsl_vector_get(probe_result.pvalue, i);
    fdr=gsl_vector_get(probe_result.fdr, i);
    range=gsl_vector_get(probe_result.range, i);
    if (pvalue<parameter.pvalue_threshold && fdr<parameter.fdr_threshold)
    {
      ofs<<probe_name<<"\t"<<pvalue<<"\t"<<fdr<<"\t"<<range<<"\t";
      for (j=0; j<data_des.group_index.size(); j++)
      {
        if (gsl_matrix_get(probe_result.average, i, j)>0)
        {
          ofs<<gsl_matrix_get(probe_result.average, i, j)<<"\t";
          fprintf(of_bgs[j], "%s\t%d\t%d\t%lf\n", ann[map_probe[probe_name]].chromosome.c_str(), ann[map_probe[probe_name]].position, ann[map_probe[probe_name]].position+1, gsl_matrix_get(probe_result.average, i, j));
        }
        else
        {
          ofs<<"-\t";
        }
      }
      ofs<<ann[map_probe[probe_name]].chromosome<<"\t"<<ann[map_probe[probe_name]].position<<"\t"<<ann[map_probe[probe_name]].annotation<<endl;
      
    }
  }
  ofs.close();
  for (i=0; i<of_bgs.size(); i++)
  {
    fclose(of_bgs[i]);
  }
  return true;
}

bool write_cgi_result(Parameter &parameter,
                      Data_description &data_des,
                      Probe_result &probe_res,
                      vector<Point> &ann,
                      map<string, size_t> &map_probe,
                      CGI_result &cgi_res,
                      CGI_description &cgi_des)
{
  fstream of_res;
  FILE *of_tmp;
  vector<FILE*> of_bg;
  string probe_name;
  vector<map<Point, double> >bgs;
  map<Point, double> emp_map;
  size_t probe_index;
  string group_name;
  string bg_file;
  size_t i;
  double aver;
  double pvalue;
  double fdr;
  Point tmp_pnt;
  time_out();
  printf("     Writing CGI result\n");
  of_res.open(parameter.cgi_test_file.c_str(), ios_base::out);
  if (!of_res.is_open())
  {
    printf("Error: cannot open file %s\n", parameter.cgi_test_file.c_str());
    return false;
  }
  for (i=0; i<data_des.group_index.size(); i++)
  {
    group_name=data_des.group_index.get(i);
    bg_file=parameter.cgi_bedgraph_prefix+"_"+group_name+".BedGraph";
    if (!(of_tmp=fopen(bg_file.c_str(), "w")))
    {
      printf("Error: cannot open file %s\n", bg_file.c_str());
      return false;
    }
    of_bg.push_back(of_tmp);
    bgs.push_back(emp_map);
  }
  size_t k;
  of_res<<"CHR\tSTART\tEND\tNAME\tPVALUE\tFDR\n";
  size_t j;
  for (i=0; i<cgi_des.cgi.size(); i++)
  {
    pvalue=gsl_vector_get(cgi_res.pvalue, i);
    fdr=gsl_vector_get(cgi_res.fdr, i);
    if (pvalue<parameter.pvalue_threshold && fdr <parameter.fdr_threshold)
    {
      of_res<<cgi_des.cgi[i].chromosome<<"\t"<<cgi_des.cgi[i].start<<"\t"<<cgi_des.cgi[i].end<<"\t"<<cgi_des.cgi[i].name<<"\t"<<pvalue<<"\t"<<fdr<<"\n";
      for (j=0; j<cgi_des.cgi_probe[i].size(); j++)
      {
        probe_name=cgi_des.cgi_probe[i][j];
        probe_index=data_des.probe_index.get(probe_name);
        tmp_pnt.chromosome=ann[map_probe[probe_name]].chromosome;
        tmp_pnt.position=ann[map_probe[probe_name]].position;
        tmp_pnt.name=probe_name;
        for (k=0; k<data_des.group_index.size(); k++)
        {
          aver=gsl_matrix_get(probe_res.average, probe_index, k);
          if (aver<1.0+1e-6)
          {
            bgs[k].insert(map<Point, double>::value_type(make_pair(tmp_pnt, aver)));
          }
        }
      }
    }
  }
  map<Point, double>::iterator itM;
  for (i=0; i<data_des.group_index.size(); i++)
  {
    for(itM=bgs[i].begin(); itM!=bgs[i].end(); itM++)
    {
      fprintf(of_bg[i], "%s\t%d\t%d\t%lf\n", itM->first.chromosome.c_str(), itM->first.position, itM->first.position+1, itM->second);
    }
  }
  of_res.close();
  for(i=0; i<of_bg.size(); i++)
  {
    fclose(of_bg[i]); 
  }
  return true;
}

bool write_dmr_result(Parameter &parameter,
                      Data_description &data_des,
                      Probe_result &probe_res,
                      vector<Point> &ann,
                      map<string, size_t> &map_probe,
                      DMR_result &dmr_res)
{
  fstream ofs;
  vector<FILE *> bgfs;
  FILE *fp;
  size_t i;
  string group_name;
  string file_name;
  ofs.open(parameter.dmr_test_file.c_str(), ios_base::out);
  if (!ofs.is_open())
  {
    printf("Error: cannot open file %s\n", parameter.dmr_test_file.c_str());
    return false;
  }
  for (i=0; i<data_des.group_index.size(); i++)
  {
    group_name=data_des.group_index.get(i);
    file_name=parameter.dmr_bedgraph_prefix+"_"+group_name+".BedGraph";
    if (!(fp=fopen(file_name.c_str(), "w")))
    {
      printf("Error: cannot open file %s\n", file_name.c_str());
      return false;
    }
    bgfs.push_back(fp);
  }
  ofs<<"CHR\tSTART\tEND\tNAME\n";
  for (i=0; i<dmr_res.dmr.size(); i++)
  {
    ofs<<dmr_res.dmr[i].chromosome<<"\t"<<dmr_res.dmr[i].start<<"\t"<<dmr_res.dmr[i].end<<"\t"<<dmr_res.dmr[i].name<<"\n";
  }
  size_t j;
  size_t k;
  double ave;
  size_t probe_index;
  Point tmp_pt;
  string probe_id;
  for(i=0; i<dmr_res.dmr.size(); i++)
  {
    for(j=0; j<dmr_res.dmr_probe[i].size(); j++)
    {
      probe_id=dmr_res.dmr_probe[i][j];
      probe_index=data_des.probe_index.get(probe_id);
      for (k=0; k<probe_res.average->size2; k++)
      {
        ave=gsl_matrix_get(probe_res.average, probe_index, k);
        if (ave<1.0+1e-6)
        {
          tmp_pt=ann[map_probe[probe_id]];
          fprintf(bgfs[k], "%s\t%d\t%d\t%lf\n", tmp_pt.chromosome.c_str(), tmp_pt.position, tmp_pt.position+1, ave);
        }
      }
    }
  }
  for (i=0; i<bgfs.size(); i++)
  {
    fclose(bgfs[i]);
  }
  ofs.close();
  return true;
}
